@@ -1,11 +1,11 @@
 /* Reading C# .resources files.
-   Copyright (C) 2003 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2006-2008, 2011 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2003.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,8 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -31,13 +30,13 @@
 #include "msgunfmt.h"
 #include "relocatable.h"
 #include "csharpexec.h"
-#include "pipe.h"
+#include "spawn-pipe.h"
 #include "wait-process.h"
+#include "read-catalog.h"
 #include "read-po.h"
 #include "message.h"
-#include "pathname.h"
+#include "concat-filename.h"
 #include "error.h"
-#include "exit.h"
 #include "gettext.h"
 
 #define _(str) gettext (str)
@@ -56,8 +55,8 @@ struct locals
 
 static bool
 execute_and_read_po_output (const char *progname,
-			    const char *prog_path, char **prog_argv,
-			    void *private_data)
+                            const char *prog_path, char **prog_argv,
+                            void *private_data)
 {
   struct locals *l = (struct locals *) private_data;
   pid_t child;
@@ -67,22 +66,23 @@ execute_and_read_po_output (const char *progname,
 
   /* Open a pipe to the C# execution engine.  */
   child = create_pipe_in (progname, prog_path, prog_argv, NULL, false,
-			  true, true, fd);
+                          true, true, fd);
 
   fp = fdopen (fd[0], "r");
   if (fp == NULL)
     error (EXIT_FAILURE, errno, _("fdopen() failed"));
 
   /* Read the message list.  */
-  l->mdlp = read_po (fp, "(pipe)", "(pipe)");
+  l->mdlp = read_catalog_stream (fp, "(pipe)", "(pipe)", &input_format_po);
 
   fclose (fp);
 
   /* Remove zombie process from process list, and retrieve exit status.  */
-  exitstatus = wait_subprocess (child, progname, false, false, true, true);
+  exitstatus =
+    wait_subprocess (child, progname, false, false, true, true, NULL);
   if (exitstatus != 0)
     error (EXIT_FAILURE, 0, _("%s subprocess failed with exit code %d"),
-	   progname, exitstatus);
+           progname, exitstatus);
 
   return false;
 }
@@ -115,12 +115,13 @@ read_resources_file (message_list_ty *mlp, const char *filename)
     gettextlibdir = relocate (LIBDIR);
 
   /* Dump the resource and retrieve the resulting output.  */
-  assembly_path = concatenated_pathname (gettextexedir, "msgunfmt.net", ".exe");
+  assembly_path =
+    xconcatenated_filename (gettextexedir, "msgunfmt.net", ".exe");
   libdirs[0] = gettextlibdir;
   if (execute_csharp_program (assembly_path, libdirs, 1,
-			      args,
-			      verbose, false,
-			      execute_and_read_po_output, &locals))
+                              args,
+                              verbose, false,
+                              execute_and_read_po_output, &locals))
     /* An error message should already have been provided.  */
     exit (EXIT_FAILURE);
 
